package com.cultural.heritage.service.good.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.CommonConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.CouponMapper;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.utils.SqlUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.yaml.snakeyaml.scanner.Constant;

import java.util.Date;

@Service
public class CouponServiceImpl extends ServiceImpl<CouponMapper, Coupon> implements CouponService {


    @Override
    public QueryWrapper<Coupon> getQueryWrapper(CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponQueryRequest.getId();
        String name = couponQueryRequest.getName();
        Date startTime = couponQueryRequest.getStartTime();
        Date endTime = couponQueryRequest.getEndTime();
        String sortField = couponQueryRequest.getSortField();
        String sortOrder = couponQueryRequest.getSortOrder();
        QueryWrapper<Coupon> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.like(StringUtils.isNotBlank(name), "name", name);
        queryWrapper.ge(ObjectUtils.isNotEmpty(startTime), "startTime", startTime);
        queryWrapper.le(ObjectUtils.isNotEmpty(endTime), "endTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }

    @Override
    public Integer verifyIsReachTheUpperLimit(Long userId, Long couponId) {
        return this.baseMapper.verifyIsReachTheUpperLimit(userId, couponId);
    }
}
